==> mule-soap-engine-1.9.4-sources.jar: org/mule/soap/internal/client/CxfClientProvider.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.soap.internal.client;

import com.google.common.collect.ImmutableList;
import org.apache.cxf.Bus;
import org.apache.cxf.binding.Binding;
import org.apache.cxf.binding.soap.interceptor.CheckFaultInterceptor;
import org.apache.cxf.binding.soap.interceptor.Soap11FaultInInterceptor;
import org.apache.cxf.binding.soap.interceptor.Soap12FaultInInterceptor;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.interceptor.Interceptor;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.PhaseInterceptor;
import org.apache.cxf.ws.addressing.WSAddressingFeature;
import org.apache.cxf.ws.addressing.impl.MAPAggregatorImpl;
import org.apache.cxf.ws.addressing.soap.MAPCodec;
import org.apache.cxf.ws.rm.RMCaptureInInterceptor;
import org.apache.cxf.ws.rm.RMCaptureOutInterceptor;
import org.apache.cxf.ws.rm.RMDeliveryInterceptor;
import org.apache.cxf.ws.rm.RMInInterceptor;
import org.apache.cxf.ws.rm.RMManager;
import org.apache.cxf.ws.rm.RMOutInterceptor;
import org.apache.cxf.ws.rm.soap.RMSoapInInterceptor;
import org.apache.cxf.ws.rm.soap.RMSoapOutInterceptor;
import org.apache.cxf.wsdl.interceptors.WrappedOutInterceptor;

import org.jetbrains.annotations.NotNull;
import org.mule.soap.api.SoapVersion;
import org.mule.soap.api.SoapWebServiceConfiguration;
import org.mule.soap.api.rm.ReliableMessagingConfiguration;
import org.mule.soap.api.security.SecurityStrategy;
import org.mule.soap.internal.interceptor.MuleRMCaptureOutInterceptor;
import org.mule.soap.internal.interceptor.OutputMtomSoapAttachmentsInterceptor;
import org.mule.soap.internal.interceptor.OutputSoapHeadersInterceptor;
import org.mule.soap.internal.interceptor.SkipAddressingInInterceptor;
import org.mule.soap.internal.interceptor.SkipAddressingOutInterceptor;
import org.mule.soap.internal.interceptor.SkipReliableMessagingInInterceptor;
import org.mule.soap.internal.interceptor.SkipReliableMessagingOutInterceptor;
import org.mule.soap.internal.interceptor.SoapActionInterceptor;
import org.mule.soap.internal.interceptor.StreamClosingInterceptor;

import org.mule.soap.internal.rm.MuleRMManager;
import org.mule.soap.internal.security.callback.CompositeCallbackHandler;
import org.mule.soap.internal.rm.store.RMStoreImp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.security.auth.callback.CallbackHandler;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

import static java.util.Collections.emptyMap;
import static org.apache.commons.lang3.StringUtils.isNotBlank;
import static org.apache.cxf.message.Message.MTOM_ENABLED;
import static org.apache.wss4j.common.ConfigurationConstants.ACTION;
import static org.apache.wss4j.common.ConfigurationConstants.PW_CALLBACK_REF;
import static org.mule.soap.api.security.SecurityStrategy.SecurityStrategyType.ALWAYS;
import static org.slf4j.LoggerFactory.getLogger;

/**
 * Object that creates CXF specific clients based on a {@link SoapWebServiceConfiguration} setting all the required CXF
 * properties.
 * <p>
 * the created client aims to be the CXF client used in the {@link AbstractSoapCxfClient}.
 *
 * @since 1.0
 */
class CxfClientProvider {

  private static final Logger logger = getLogger(CxfClientProvider.class);

  private final CxfClientFactory factory = new CxfClientFactory();
  private ClassLoader fipsSensitiveClassLoader;

  Client getClient(SoapWebServiceConfiguration configuration, ClassLoader classLoader) {
    initClassLoader(classLoader);
    boolean isMtom = configuration.isMtomEnabled();
    String address = configuration.getAddress();
    SoapVersion version = configuration.getVersion();
    Client client = factory.createClient(address, version.getNumber());
    addSecurityInterceptors(client, configuration.getSecurities());
    addRequestInterceptors(client);
    addResponseInterceptors(client, isMtom);
    addAddressingInterceptors(client);
    addReliableMessagingInterceptors(client);
    client.getEndpoint().put(MTOM_ENABLED, isMtom);
    configuration.getReliableMessagingConfiguration()
        .ifPresent(config -> configureReliableMessaging(client, config));
    removeUnnecessaryCxfInterceptors(client);
    return client;
  }

  void initClassLoader(ClassLoader classLoader) {
    this.fipsSensitiveClassLoader = classLoader;
  }

  private void addSecurityInterceptors(Client client, List<SecurityStrategy> securityStrategies) {
    Map<String, Object> requestProps =
        buildSecurityProperties(securityStrategies, SecurityStrategy.SecurityStrategyType.OUTGOING);
    if (!requestProps.isEmpty() && isNotBlank((String) requestProps.get(ACTION))) {
      try {
        Interceptor interceptor = createWss4JOutInterceptor(requestProps);
        client.getOutInterceptors().add(interceptor);
      } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | NoSuchMethodException
          | InvocationTargetException e) {
        logger.error("Cannot instantiate WSS4JOutInterceptor class: " + e);
        throw new RuntimeException(e);
      }
    }

    Map<String, Object> responseProps =
        buildSecurityProperties(securityStrategies, SecurityStrategy.SecurityStrategyType.INCOMING);
    if (!responseProps.isEmpty() && isNotBlank((String) responseProps.get(ACTION))) {
      try {
        Interceptor interceptor = createWss4JInInterceptor(responseProps);
        client.getInInterceptors().add(interceptor);
      } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | InvocationTargetException
          | NoSuchMethodException e) {
        logger.error("Cannot instantiate WSS4JInInterceptor class: " + e);
        throw new RuntimeException(e);
      }
    }
  }

  @NotNull
  public Interceptor createWss4JOutInterceptor(Map<String, Object> requestProps) throws ClassNotFoundException,
      InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
    logger.debug("Creation of org.apache.cxf.ws.security.wss4j.WSS4JOutInterceptor instance");
    Class<?> wSS4JOutInterceptorClass =
        fipsSensitiveClassLoader.loadClass("org.apache.cxf.ws.security.wss4j.WSS4JOutInterceptor");
    Interceptor interceptor =
        (Interceptor) wSS4JOutInterceptorClass.getDeclaredConstructor(Map.class).newInstance(requestProps);
    return interceptor;
  }

  @NotNull
  public Interceptor createWss4JInInterceptor(Map<String, Object> responseProps) throws ClassNotFoundException,
      InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
    logger.debug("Creation of org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor instance");
    Class<?> wSS4JInInterceptorClass =
        fipsSensitiveClassLoader.loadClass("org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor");
    Interceptor interceptor =
        (Interceptor) wSS4JInInterceptorClass.getDeclaredConstructor(Map.class).newInstance(responseProps);
    return interceptor;
  }

  private Map<String, Object> buildSecurityProperties(List<SecurityStrategy> strategies,
                                                      SecurityStrategy.SecurityStrategyType type) {
    if (strategies.isEmpty()) {
      return emptyMap();
    }

    Map<String, Object> props = new HashMap<>();
    StringJoiner actionsJoiner = new StringJoiner(" ");

    ImmutableList.Builder<CallbackHandler> callbackHandlersBuilder = ImmutableList.builder();
    strategies.stream()
        .filter(strategy -> strategy.securityType().equals(type) || strategy.securityType().equals(ALWAYS))
        .forEach(securityStrategy -> {
          props.putAll(securityStrategy.buildSecurityProperties());
          if (isNotBlank(securityStrategy.securityAction())) {
            actionsJoiner.add(securityStrategy.securityAction());
          }
          securityStrategy.buildPasswordCallbackHandler().ifPresent(callbackHandlersBuilder::add);
        });

    List<CallbackHandler> handlers = callbackHandlersBuilder.build();
    if (!handlers.isEmpty()) {
      props.put(PW_CALLBACK_REF, new CompositeCallbackHandler(handlers));
    }

    String actions = actionsJoiner.toString();
    if (isNotBlank(actions)) {
      // the list of actions is passed as a String with the action names separated by a black space.
      props.put(ACTION, actions);
    }

    // This Map needs to be mutable, cxf will add properties if needed.
    return props;
  }

  private void addRequestInterceptors(Client client) {
    List<Interceptor<? extends Message>> outInterceptors = client.getOutInterceptors();
    outInterceptors.add(new SoapActionInterceptor());
  }

  private void addResponseInterceptors(Client client, boolean mtomEnabled) {
    List<Interceptor<? extends Message>> inInterceptors = client.getInInterceptors();
    inInterceptors.add(new StreamClosingInterceptor());
    inInterceptors.add(new CheckFaultInterceptor());
    inInterceptors.add(new OutputSoapHeadersInterceptor());
    inInterceptors.add(new SoapActionInterceptor());
    inInterceptors.add(new OutputMtomSoapAttachmentsInterceptor());
  }

  private void removeUnnecessaryCxfInterceptors(Client client) {
    Binding binding = client.getEndpoint().getBinding();
    removeInterceptor(binding.getOutInterceptors(), WrappedOutInterceptor.class.getName());
    removeInterceptor(binding.getInInterceptors(), Soap11FaultInInterceptor.class.getName());
    removeInterceptor(binding.getInInterceptors(), Soap12FaultInInterceptor.class.getName());
    removeInterceptor(binding.getInInterceptors(), CheckFaultInterceptor.class.getName());
  }

  private void removeInterceptor(List<Interceptor<? extends Message>> inInterceptors, String name) {
    inInterceptors.removeIf(i -> i instanceof PhaseInterceptor && ((PhaseInterceptor) i).getId().equals(name));
  }

  private void addAddressingInterceptors(Client client) {

    WSAddressingFeature feature = new WSAddressingFeature();
    MAPCodec mapCodec = MAPCodec.getInstance(client.getBus());
    MAPAggregatorImpl mapAggregator = new MAPAggregatorImpl();
    SkipAddressingInInterceptor inInterceptor = new SkipAddressingInInterceptor(mapCodec, mapAggregator);
    SkipAddressingOutInterceptor outInterceptor = new SkipAddressingOutInterceptor(mapCodec, mapAggregator);
    mapAggregator.setAllowDuplicates(feature.isAllowDuplicates());
    mapAggregator.setUsingAddressingAdvisory(feature.isUsingAddressingAdvisory());
    mapAggregator.setAddressingRequired(feature.isAddressingRequired());
    if (feature.getMessageIdCache() != null) {
      mapAggregator.setMessageIdCache(feature.getMessageIdCache());
    }

    client.getInInterceptors().add(mapAggregator);
    client.getInInterceptors().add(mapCodec);
    client.getInInterceptors().add(inInterceptor);

    client.getOutInterceptors().add(mapAggregator);
    client.getOutInterceptors().add(mapCodec);
    client.getOutInterceptors().add(outInterceptor);

    client.getInFaultInterceptors().add(mapAggregator);
    client.getInFaultInterceptors().add(mapCodec);
    client.getInFaultInterceptors().add(inInterceptor);

    client.getOutFaultInterceptors().add(mapAggregator);
    client.getOutFaultInterceptors().add(mapCodec);
    client.getOutFaultInterceptors().add(outInterceptor);
  }

  private void addReliableMessagingInterceptors(Client client) {
    Bus bus = client.getBus();
    MuleRMManager manager = new MuleRMManager(bus.getExtension(RMManager.class));
    manager.getSourcePolicy().getSequenceTerminationPolicy().setTerminateOnShutdown(false);
    bus.setExtension(manager, RMManager.class);

    RMInInterceptor rmLogicalIn = new RMInInterceptor();
    RMOutInterceptor rmLogicalOut = new RMOutInterceptor();
    RMDeliveryInterceptor rmDelivery = new RMDeliveryInterceptor();
    RMSoapOutInterceptor rmOutCodec = new RMSoapOutInterceptor();
    RMSoapInInterceptor rmInCodec = new RMSoapInInterceptor();
    RMCaptureInInterceptor rmCaptureIn = new RMCaptureInInterceptor();
    RMCaptureOutInterceptor rmCaptureOut = new MuleRMCaptureOutInterceptor();
    SkipReliableMessagingOutInterceptor outInterceptor =
        new SkipReliableMessagingOutInterceptor(rmLogicalOut, rmOutCodec, rmCaptureOut);
    SkipReliableMessagingInInterceptor inInterceptor =
        new SkipReliableMessagingInInterceptor(rmLogicalIn, rmDelivery, rmInCodec, rmCaptureIn);

    rmLogicalIn.setBus(bus);
    rmLogicalOut.setBus(bus);
    rmDelivery.setBus(bus);
    rmCaptureIn.setBus(bus);
    rmCaptureOut.setBus(bus);

    client.getInInterceptors().add(rmLogicalIn);
    client.getInInterceptors().add(rmInCodec);
    client.getInInterceptors().add(rmDelivery);
    client.getInInterceptors().add(rmCaptureIn);
    client.getInInterceptors().add(inInterceptor);

    client.getOutInterceptors().add(rmLogicalOut);
    client.getOutInterceptors().add(rmOutCodec);
    client.getOutInterceptors().add(rmCaptureOut);
    client.getOutInterceptors().add(outInterceptor);

    client.getInFaultInterceptors().add(rmLogicalIn);
    client.getInFaultInterceptors().add(rmInCodec);
    client.getInFaultInterceptors().add(rmDelivery);
    client.getInFaultInterceptors().add(inInterceptor);

    client.getOutFaultInterceptors().add(rmLogicalOut);
    client.getOutFaultInterceptors().add(rmOutCodec);
    client.getOutFaultInterceptors().add(rmCaptureOut);
    client.getOutFaultInterceptors().add(outInterceptor);
  }

  private void configureReliableMessaging(Client client, ReliableMessagingConfiguration config) {
    RMManager manager = client.getBus().getExtension(RMManager.class);
    manager.setStore(new RMStoreImp(config.getStore()));
    manager.clientCreated(client);
  }
}

==> mule-soap-engine-1.10.0-sources.jar: org/mule/soap/internal/client/CxfClientProvider.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.soap.internal.client;

import com.google.common.collect.ImmutableList;
import org.apache.cxf.Bus;
import org.apache.cxf.binding.Binding;
import org.apache.cxf.binding.soap.interceptor.CheckFaultInterceptor;
import org.apache.cxf.binding.soap.interceptor.Soap11FaultInInterceptor;
import org.apache.cxf.binding.soap.interceptor.Soap12FaultInInterceptor;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.interceptor.Interceptor;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.PhaseInterceptor;
import org.apache.cxf.ws.addressing.WSAddressingFeature;
import org.apache.cxf.ws.addressing.impl.MAPAggregatorImpl;
import org.apache.cxf.ws.addressing.soap.MAPCodec;
import org.apache.cxf.ws.rm.RMCaptureInInterceptor;
import org.apache.cxf.ws.rm.RMCaptureOutInterceptor;
import org.apache.cxf.ws.rm.RMDeliveryInterceptor;
import org.apache.cxf.ws.rm.RMInInterceptor;
import org.apache.cxf.ws.rm.RMManager;
import org.apache.cxf.ws.rm.RMOutInterceptor;
import org.apache.cxf.ws.rm.soap.RMSoapInInterceptor;
import org.apache.cxf.ws.rm.soap.RMSoapOutInterceptor;
import org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor;
import org.apache.cxf.ws.security.wss4j.WSS4JOutInterceptor;
import org.apache.cxf.wsdl.interceptors.WrappedOutInterceptor;

import org.mule.soap.api.SoapVersion;
import org.mule.soap.api.SoapWebServiceConfiguration;
import org.mule.soap.api.rm.ReliableMessagingConfiguration;
import org.mule.soap.api.security.SecurityStrategy;
import org.mule.soap.internal.interceptor.MuleRMCaptureOutInterceptor;
import org.mule.soap.internal.interceptor.OutputMtomSoapAttachmentsInterceptor;
import org.mule.soap.internal.interceptor.OutputSoapHeadersInterceptor;
import org.mule.soap.internal.interceptor.SkipAddressingInInterceptor;
import org.mule.soap.internal.interceptor.SkipAddressingOutInterceptor;
import org.mule.soap.internal.interceptor.SkipReliableMessagingInInterceptor;
import org.mule.soap.internal.interceptor.SkipReliableMessagingOutInterceptor;
import org.mule.soap.internal.interceptor.SoapActionInterceptor;
import org.mule.soap.internal.interceptor.StreamClosingInterceptor;

import org.mule.soap.internal.rm.MuleRMManager;
import org.mule.soap.internal.security.callback.CompositeCallbackHandler;
import org.mule.soap.internal.rm.store.RMStoreImp;
import org.slf4j.Logger;

import javax.security.auth.callback.CallbackHandler;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

import static java.util.Collections.emptyMap;
import static org.apache.commons.lang3.StringUtils.isNotBlank;
import static org.apache.cxf.message.Message.MTOM_ENABLED;
import static org.apache.wss4j.common.ConfigurationConstants.ACTION;
import static org.apache.wss4j.common.ConfigurationConstants.PW_CALLBACK_REF;
import static org.mule.soap.api.security.SecurityStrategy.SecurityStrategyType.ALWAYS;
import static org.slf4j.LoggerFactory.getLogger;

/**
 * Object that creates CXF specific clients based on a {@link SoapWebServiceConfiguration} setting all the required CXF
 * properties.
 * <p>
 * the created client aims to be the CXF client used in the {@link AbstractSoapCxfClient}.
 *
 * @since 1.0
 */
class CxfClientProvider {

  private final CxfClientFactory factory = new CxfClientFactory();

  Client getClient(SoapWebServiceConfiguration configuration) {
    boolean isMtom = configuration.isMtomEnabled();
    String address = configuration.getAddress();
    SoapVersion version = configuration.getVersion();
    Client client = factory.createClient(address, version.getNumber());
    addSecurityInterceptors(client, configuration.getSecurities());
    addRequestInterceptors(client);
    addResponseInterceptors(client, isMtom);
    addAddressingInterceptors(client);
    addReliableMessagingInterceptors(client);
    client.getEndpoint().put(MTOM_ENABLED, isMtom);
    configuration.getReliableMessagingConfiguration()
        .ifPresent(config -> configureReliableMessaging(client, config));
    removeUnnecessaryCxfInterceptors(client);
    return client;
  }

  private void addSecurityInterceptors(Client client, List<SecurityStrategy> securityStrategies) {
    Map<String, Object> requestProps =
        buildSecurityProperties(securityStrategies, SecurityStrategy.SecurityStrategyType.OUTGOING);
    if (!requestProps.isEmpty() && isNotBlank((String) requestProps.get(ACTION))) {
      client.getOutInterceptors().add(new WSS4JOutInterceptor(requestProps));
    }

    Map<String, Object> responseProps =
        buildSecurityProperties(securityStrategies, SecurityStrategy.SecurityStrategyType.INCOMING);
    if (!responseProps.isEmpty() && isNotBlank((String) responseProps.get(ACTION))) {
      client.getInInterceptors().add(new WSS4JInInterceptor(responseProps));
    }
  }

  private Map<String, Object> buildSecurityProperties(List<SecurityStrategy> strategies,
                                                      SecurityStrategy.SecurityStrategyType type) {
    if (strategies.isEmpty()) {
      return emptyMap();
    }

    Map<String, Object> props = new HashMap<>();
    StringJoiner actionsJoiner = new StringJoiner(" ");

    ImmutableList.Builder<CallbackHandler> callbackHandlersBuilder = ImmutableList.builder();
    strategies.stream()
        .filter(strategy -> strategy.securityType().equals(type) || strategy.securityType().equals(ALWAYS))
        .forEach(securityStrategy -> {
          props.putAll(securityStrategy.buildSecurityProperties());
          if (isNotBlank(securityStrategy.securityAction())) {
            actionsJoiner.add(securityStrategy.securityAction());
          }
          securityStrategy.buildPasswordCallbackHandler().ifPresent(callbackHandlersBuilder::add);
        });

    List<CallbackHandler> handlers = callbackHandlersBuilder.build();
    if (!handlers.isEmpty()) {
      props.put(PW_CALLBACK_REF, new CompositeCallbackHandler(handlers));
    }

    String actions = actionsJoiner.toString();
    if (isNotBlank(actions)) {
      // the list of actions is passed as a String with the action names separated by a black space.
      props.put(ACTION, actions);
    }

    // This Map needs to be mutable, cxf will add properties if needed.
    return props;
  }

  private void addRequestInterceptors(Client client) {
    List<Interceptor<? extends Message>> outInterceptors = client.getOutInterceptors();
    outInterceptors.add(new SoapActionInterceptor());
  }

  private void addResponseInterceptors(Client client, boolean mtomEnabled) {
    List<Interceptor<? extends Message>> inInterceptors = client.getInInterceptors();
    inInterceptors.add(new StreamClosingInterceptor());
    inInterceptors.add(new CheckFaultInterceptor());
    inInterceptors.add(new OutputSoapHeadersInterceptor());
    inInterceptors.add(new SoapActionInterceptor());
    inInterceptors.add(new OutputMtomSoapAttachmentsInterceptor());
  }

  private void removeUnnecessaryCxfInterceptors(Client client) {
    Binding binding = client.getEndpoint().getBinding();
    removeInterceptor(binding.getOutInterceptors(), WrappedOutInterceptor.class.getName());
    removeInterceptor(binding.getInInterceptors(), Soap11FaultInInterceptor.class.getName());
    removeInterceptor(binding.getInInterceptors(), Soap12FaultInInterceptor.class.getName());
    removeInterceptor(binding.getInInterceptors(), CheckFaultInterceptor.class.getName());
  }

  private void removeInterceptor(List<Interceptor<? extends Message>> inInterceptors, String name) {
    inInterceptors.removeIf(i -> i instanceof PhaseInterceptor && ((PhaseInterceptor) i).getId().equals(name));
  }

  private void addAddressingInterceptors(Client client) {

    WSAddressingFeature feature = new WSAddressingFeature();
    MAPCodec mapCodec = MAPCodec.getInstance(client.getBus());
    MAPAggregatorImpl mapAggregator = new MAPAggregatorImpl();
    SkipAddressingInInterceptor inInterceptor = new SkipAddressingInInterceptor(mapCodec, mapAggregator);
    SkipAddressingOutInterceptor outInterceptor = new SkipAddressingOutInterceptor(mapCodec, mapAggregator);
    mapAggregator.setAllowDuplicates(feature.isAllowDuplicates());
    mapAggregator.setUsingAddressingAdvisory(feature.isUsingAddressingAdvisory());
    mapAggregator.setAddressingRequired(feature.isAddressingRequired());
    if (feature.getMessageIdCache() != null) {
      mapAggregator.setMessageIdCache(feature.getMessageIdCache());
    }

    client.getInInterceptors().add(mapAggregator);
    client.getInInterceptors().add(mapCodec);
    client.getInInterceptors().add(inInterceptor);

    client.getOutInterceptors().add(mapAggregator);
    client.getOutInterceptors().add(mapCodec);
    client.getOutInterceptors().add(outInterceptor);

    client.getInFaultInterceptors().add(mapAggregator);
    client.getInFaultInterceptors().add(mapCodec);
    client.getInFaultInterceptors().add(inInterceptor);

    client.getOutFaultInterceptors().add(mapAggregator);
    client.getOutFaultInterceptors().add(mapCodec);
    client.getOutFaultInterceptors().add(outInterceptor);
  }

  private void addReliableMessagingInterceptors(Client client) {
    Bus bus = client.getBus();
    MuleRMManager manager = new MuleRMManager(bus.getExtension(RMManager.class));
    manager.getSourcePolicy().getSequenceTerminationPolicy().setTerminateOnShutdown(false);
    bus.setExtension(manager, RMManager.class);

    RMInInterceptor rmLogicalIn = new RMInInterceptor();
    RMOutInterceptor rmLogicalOut = new RMOutInterceptor();
    RMDeliveryInterceptor rmDelivery = new RMDeliveryInterceptor();
    RMSoapOutInterceptor rmOutCodec = new RMSoapOutInterceptor();
    RMSoapInInterceptor rmInCodec = new RMSoapInInterceptor();
    RMCaptureInInterceptor rmCaptureIn = new RMCaptureInInterceptor();
    RMCaptureOutInterceptor rmCaptureOut = new MuleRMCaptureOutInterceptor();
    SkipReliableMessagingOutInterceptor outInterceptor =
        new SkipReliableMessagingOutInterceptor(rmLogicalOut, rmOutCodec, rmCaptureOut);
    SkipReliableMessagingInInterceptor inInterceptor =
        new SkipReliableMessagingInInterceptor(rmLogicalIn, rmDelivery, rmInCodec, rmCaptureIn);

    rmLogicalIn.setBus(bus);
    rmLogicalOut.setBus(bus);
    rmDelivery.setBus(bus);
    rmCaptureIn.setBus(bus);
    rmCaptureOut.setBus(bus);

    client.getInInterceptors().add(rmLogicalIn);
    client.getInInterceptors().add(rmInCodec);
    client.getInInterceptors().add(rmDelivery);
    client.getInInterceptors().add(rmCaptureIn);
    client.getInInterceptors().add(inInterceptor);

    client.getOutInterceptors().add(rmLogicalOut);
    client.getOutInterceptors().add(rmOutCodec);
    client.getOutInterceptors().add(rmCaptureOut);
    client.getOutInterceptors().add(outInterceptor);

    client.getInFaultInterceptors().add(rmLogicalIn);
    client.getInFaultInterceptors().add(rmInCodec);
    client.getInFaultInterceptors().add(rmDelivery);
    client.getInFaultInterceptors().add(inInterceptor);

    client.getOutFaultInterceptors().add(rmLogicalOut);
    client.getOutFaultInterceptors().add(rmOutCodec);
    client.getOutFaultInterceptors().add(rmCaptureOut);
    client.getOutFaultInterceptors().add(outInterceptor);
  }

  private void configureReliableMessaging(Client client, ReliableMessagingConfiguration config) {
    RMManager manager = client.getBus().getExtension(RMManager.class);
    manager.setStore(new RMStoreImp(config.getStore()));
    manager.clientCreated(client);
  }
}
